#!/bin/env node
"use strict";

// src/setup/index.ts
var import_commander = require("commander");
var import_crypto = require("crypto");
var import_aws_cdk_lib = require("aws-cdk-lib");
var import_client_secrets_manager3 = require("@aws-sdk/client-secrets-manager");

// src/lambdas/key-rotation.ts
var import_client_secrets_manager2 = require("@aws-sdk/client-secrets-manager");

// src/lambdas/entra-sdk-client.ts
var import_microsoft_graph_client = require("@microsoft/microsoft-graph-client");

// src/lambdas/get-access-token.ts
var getAccessToken = async ({
  tenantId: tenantId2,
  clientId: clientId2,
  clientSecret: clientSecret2,
  scopes
}) => {
  const res = await fetch(`https://login.microsoftonline.com/${tenantId2}/oauth2/v2.0/token`, {
    headers: {
      "content-type": "application/x-www-form-urlencoded"
    },
    method: "post",
    body: new URLSearchParams({
      client_id: clientId2,
      scope: scopes?.length ? scopes.join(" ") : "https://graph.microsoft.com/.default",
      grant_type: "client_credentials",
      client_secret: clientSecret2
    }).toString()
  });
  if (!res.ok) {
    throw new Error(`microsoft /token call failed with ${res.status}: ${await res.text()}`);
  }
  const { access_token } = await res.json();
  if (!access_token) {
    throw new Error("no access token returned from /token call");
  }
  return access_token;
};

// src/lambdas/entra-sdk-client.ts
var import_client_secrets_manager = require("@aws-sdk/client-secrets-manager");
var objectIsBootstrapClientInfo = (obj) => {
  if (!obj.validForMsStr || !obj.clientId || !obj.secretText || !obj.keyId || !obj.tenantId) {
    return false;
  }
  return true;
};
var getBootstrapClientInfo = async () => {
  const bootstrapClientSecretId = process.env.BOOTSTRAP_CLIENT_SECRET_ID;
  if (!bootstrapClientSecretId) {
    throw new Error("env var BOOTSTRAP_CLIENT_SECRET_ID missing");
  }
  const secretClient = new import_client_secrets_manager.SecretsManagerClient();
  const secret = await secretClient.send(
    new import_client_secrets_manager.GetSecretValueCommand({
      SecretId: bootstrapClientSecretId
    })
  );
  if (!secret.SecretString) {
    throw new Error("missing bootstrap secret string value");
  }
  const obj = JSON.parse(secret.SecretString);
  if (!objectIsBootstrapClientInfo(obj)) {
    throw new Error("invalid bootstrap secret json object, missing properties");
  }
  return obj;
};
var getBootstrapClientInfoPromise;
var client = import_microsoft_graph_client.Client.initWithMiddleware({
  defaultVersion: "v1.0",
  authProvider: {
    async getAccessToken(authenticationProviderOptions) {
      if (!getBootstrapClientInfoPromise) {
        getBootstrapClientInfoPromise = getBootstrapClientInfo();
      }
      const bootstrapClientInfo = await getBootstrapClientInfoPromise;
      return await getAccessToken({
        clientId: bootstrapClientInfo.clientId,
        clientSecret: bootstrapClientInfo.secretText,
        tenantId: bootstrapClientInfo.tenantId,
        scopes: authenticationProviderOptions?.scopes
      });
    }
  }
});

// src/lambdas/entra-app-key.ts
var getEntraAppKey = async (appId, keyId2) => {
  const app = await client.api(`/applications(appId='${appId}')`).get();
  const key = app.passwordCredentials?.find((key2) => key2.keyId === keyId2);
  return key;
};
var createEntraAppKey = async (appId, keyInfo, validForMsStr) => {
  const validForMs = parseInt(validForMsStr, 10);
  if (Number.isNaN(validForMs)) {
    throw new Error(`Unable to parse validForMs tag value "${validForMsStr}", result was NaN`);
  }
  const endDateTime = /* @__PURE__ */ new Date();
  endDateTime.setUTCMilliseconds(endDateTime.getUTCMilliseconds() + validForMs);
  const res = await client.api(`/applications(appId='${appId}')/addPassword`).post({
    passwordCredential: {
      ...keyInfo,
      endDateTime: endDateTime.toISOString()
    }
  });
  return res;
};

// src/lambdas/key-rotation.ts
var rotateEntraKey = async (appId, keyId2, validForMsStr) => {
  if (!appId || !keyId2 || !validForMsStr) {
    throw new Error(`Required secret values, only found: ${JSON.stringify({ appId, keyId: keyId2, validForMsStr })}`);
  }
  const existing = await getEntraAppKey(appId, keyId2);
  if (!existing) {
    throw new Error("Existing key not found in Entra ID");
  }
  console.log(`Found key to rotate in Entra ID: "${existing.displayName}"`);
  const [origName, rotationNumStr] = existing.displayName?.split("- rotation ") ?? [];
  const rotationNum = rotationNumStr ? parseInt(rotationNumStr, 10) : 0;
  const newName = `${origName} - ${Number.isNaN(rotationNum) ? 1 : rotationNum + 1}`;
  const newKey = await createEntraAppKey(
    appId,
    {
      displayName: newName
    },
    validForMsStr
  );
  return newKey;
};
var getNewSecretString = async (client2, secretId) => {
  const current = await client2.send(
    new import_client_secrets_manager2.GetSecretValueCommand({
      SecretId: secretId,
      VersionStage: "AWSCURRENT"
    })
  );
  const { clientId: clientId2, keyId: keyId2, validForMsStr, tenantId: tenantId2 } = JSON.parse(current.SecretString ?? "{}");
  const newKey = await rotateEntraKey(clientId2, keyId2, validForMsStr);
  if (!newKey.secretText || !newKey.keyId) {
    throw new Error("new key missing required properties");
  }
  const newObj = {
    clientId: clientId2,
    validForMsStr,
    tenantId: tenantId2,
    secretText: newKey.secretText,
    keyId: newKey.keyId
  };
  return JSON.stringify(newObj);
};

// src/setup/index.ts
var program = new import_commander.Command();
program.option("--clientId <CLIENT ID>").option("--clientSecret <CLIENT SECRET>").option("--tenantId <TENANT ID>").option("--keyId <KEY ID>");
program.parse();
var { clientId, clientSecret, tenantId, keyId } = program.opts();
if (!clientId || !clientSecret || !tenantId || !keyId) {
  throw new Error("missing option");
}
var go = async () => {
  console.log("Creating secret...");
  const ssm = new import_client_secrets_manager3.SecretsManagerClient({ region: process.env.AWS_REGION });
  const obj = {
    validForMsStr: import_aws_cdk_lib.Duration.days(30).toMilliseconds().toString(),
    clientId,
    secretText: clientSecret,
    keyId,
    tenantId
  };
  const secret = await ssm.send(
    new import_client_secrets_manager3.CreateSecretCommand({
      Name: `entra-bootstrap-${(0, import_crypto.randomUUID)().split("-")[0]}-secret`,
      SecretString: JSON.stringify(obj)
    })
  );
  if (!secret.ARN) {
    throw new Error("Created secret has no ARN");
  }
  console.log(`Created secret ${secret.ARN}`);
  console.log("Rotating...");
  process.env.BOOTSTRAP_CLIENT_SECRET_ID = secret.ARN;
  try {
    await ssm.send(
      new import_client_secrets_manager3.PutSecretValueCommand({
        SecretId: secret.ARN,
        SecretString: await getNewSecretString(ssm, secret.ARN)
      })
    );
  } catch (e) {
    console.error("Rotation failed, deleting secret...");
    await ssm.send(
      new import_client_secrets_manager3.DeleteSecretCommand({
        SecretId: secret.ARN,
        ForceDeleteWithoutRecovery: true
      })
    );
    console.error("Deleted secret, throwing original error...");
    throw e;
  }
  console.log("\u2728 Complete");
  console.log(`Entra ID Boostrap Secret ARN: "${secret.ARN}"`);
  console.log(
    "Now create a stack which:\n	1. Wraps the secret with EntraSelfRotatingKey so it's regularly rotated.\n	2. Exports the secret arn so other stacks can use it."
  );
};
go().catch((e) => {
  console.error(e);
  process.exit(1);
});
